util: Optimize path_encode
PATCH 1/1 util: Optimize path_encode
strncpy() shows up in profiling.
Since we already know the length, use memcpy() instead.
Measured 2% _user_ cpu time reduction on EeePC coldplug
udev/lib/libudev-util.c +5 -3
@@ -114,7 +114,6 @@ size_t util_path_encode(char *s, size_t len)char t[(len *3 )+1 ];size_t i, j; - t[0 ] ='\0' ;for (i =0 , j =0 ; s[i] !='\0' ; i++) {if (s[i] ='/' ) {memcpy (&t[j],"\\x2f" ,4 );
@@ -127,8 +126,11 @@ size_t util_path_encode(char *s, size_t len) j++; } } - t[j] ='\0' ; -strncpy (s, t, len); +if (len =0 ) +return j; + i = (j < len -1 ) ? j : len -1 ; +memcpy (s, t, i); + s[i] ='\0' ;return j; }
Kay Sievers
AppliedQuoted 4 lines · Alan Jenkins
strncpy()shows up in profiling.
Since we already know the length, usememcpy()instead.Measured 2% _user_ cpu time reduction on EeePC coldplug
Applied.
Thanks,
Kay
Quoted 5 lines · Kay Sievers
Quoted 3 lines · Alan Jenkins
strncpy()shows up in profiling.
Since we already know the length, usememcpy()instead.
Quoted 8 lines
Quoted 2 lines
Measured 2% _user_ cpu time reduction on EeePC coldplug
Applied.
Thanks,
Kay
Ah... just realised it wasn't the normal C string length inefficiency. The problem was strncpy() doesn't stop after writing the terminating NUL; by definition it goes on to zero the entire buffer.
I spy another use of strncpy in udev_device_add_property_from_string(), which is responsible for another ~1% user cpu time...
BTW I'm still polishing threaded udevd in my spare time. It has 4 TODOs; I will post it soon anyway, but I need to write a few explanatory words first. I have coarse-grained patch descriptions, but there are a couple of obvious questions I would like to pre-answer :).